% create_20news(nWords)
%
% creates a dataset with 'nWords' from the 20 newsgroups
% dataset. This function first calls rainbow to parse the data and
% converts the result to matlab format. The 'nWords' words with the
% highest mutual information gain with the class variable are used
% for the corpus.
function create_20news(nWords)

if nargin < 1
  help create_20news
  error('too few input arguments');
end

%
% launch rainbow
%
prefix = ['20news_w' num2str(nWords)];

dict_file = ['/tmp/' prefix '_dict'];
count_file = ['/tmp/' prefix '_counts'];

data_dir = '/kyb/agbs/pgehler/docretrieval/20Newsgroups-orig/20Newsgroups-bydata';


system(['~/docretrieval/bow/rainbow -d /tmp/newsdata -T ' num2str(nWords) '-i -h --use-stemming --use-stoplist --shortest-word=3 ' data_dir '/*']);
system(['~/docretrieval/bow/rainbow -d /tmp/newsdata -B=sin | sort +0 -1 > ' count_file]);
system(['~/docretrieval/bow/rainbow -d /tmp/newsdata -B=wa | head -1 > ' dict_file]);

fid = fopen(dict_file);
currline = fgetl(fid);
fclose(fid);
t0 = findstr(currline,' ');

[word dummy] = strread(currline(t0(3)+1:end),'%s %d');
clear dummy;

save([prefix '_dict'],'word');

% allocate space for the corpus
Nwords = length(word);
[dummy,Ndocuments] = system(['find ' data_dir ' -type f | wc -l']);
Ndocuments = str2num(Ndocuments);
fprintf('allocating space for %d documents\n',Ndocuments);
counts = sparse(Nwords,Ndocuments);
labels = zeros(1,Ndocuments);
labels_ll = zeros(1,Ndocuments);


toplevel_labels = {'comp','rec','sci','talk','alt','misc','soc'};
lowlevel_labels = {'alt.atheism','comp.graphics','comp.os.ms-windows.misc','comp.sys.ibm.pc.hardware','comp.sys.mac.hardware','comp.windows.x','misc.forsale','rec.autos','rec.motorcycles','rec.sport.baseball','rec.sport.hockey','sci.crypt','sci.electronics','sci.med','sci.space','soc.religion.christian','talk.politics.guns','talk.politics.mideast','talk.politics.misc','talk.religion.misc'};


fid = fopen(count_file);

cnt = 1;
emptydoc = 0;
while 1
    currline = fgetl(fid);
    if ~ischar(currline), break, end
    
    t0 = findstr(currline,' ');
    docfilename = currline(1:t0(1)-1);
    groupname = currline(t0(1)+1:t0(2)-1);
    tt = strfind(groupname,'.');
    [dummy,labels(cnt)] = max(strcmp(groupname(1:tt(1)-1),toplevel_labels));
    [dummy,labels_ll(cnt)] = max(strcmp(groupname,lowlevel_labels));

    try
        [wordid wordcount] = strread(currline(t0(3)+1:end),'%d %d');

        if ~prod(size(wordcount)) | ~prod(size(wordid))
	    emptydoc = emptydoc+1;
	%    fprintf('       empty document no %d\r',emptydoc);
	    continue;
	    % cnt is not increased so labels(cnt) will be overwritten
	end
	counts(wordid+1,cnt) = wordcount;
        cnt = cnt+1;
    catch
        emptydoc = emptydoc+1;
%        fprintf('       empty document no %d\r',emptydoc);
       % counts(:,end) = [];
        %labels(end) = [];
    end
    
    if ~mod(cnt,100), fprintf('%d of %d\r',cnt,Ndocuments);end
end
fclose(fid);
while 1, if sum(counts(:,end)) ~= 0, break;end; counts(:,end)=[];labels(end)=[];labels_ll(end)=[];end

fprintf('\nremaining corpus size %d\n',length(counts));

save([prefix '_counts'],'counts','labels','labels_ll','toplevel_labels','lowlevel_labels');
